from django.contrib.auth import get_user_model
from django.conf import settings
from django.utils.decorators import method_decorator
from django.utils.translation import ugettext_lazy as _
from django.views.decorators.debug import sensitive_post_parameters

from rest_framework.response import Response
from rest_framework import status, viewsets, permissions, generics, views, exceptions

try:
    from rest_auth.registration.views import RegisterView
    from allauth.account import app_settings as allauth_settings
    from allauth.account.models import EmailAddress, EmailConfirmationHMAC
except ImportError:
    raise ImportError("you need to install django-allauth package")

try:
    from rest_auth.app_settings import (UserDetailsSerializer, PasswordResetConfirmSerializer, create_token)
    from rest_auth.utils import jwt_encode
    from rest_auth.registration.views import VerifyEmailView
    from rest_auth.views import LoginView, LogoutView, PasswordChangeView, PasswordResetConfirmView
except ImportError:
    raise ImportError("you need to install django-rest-auth package")

from .tasks import send_register_mail, send_reset_password_email

User = get_user_model()

sensitive_post_parameters_m = method_decorator(
    sensitive_post_parameters('password1', 'password2')
)


class RegisterUserView(RegisterView):

    def perform_create(self, serializer):
        user = serializer.save(self.request)
        if getattr(settings, 'REST_USE_JWT', False):
            self.token = jwt_encode(user)
        else:
            create_token(self.token_model, user, serializer)
        email = EmailAddress.objects.get(user=user, email=user.email)
        confirmation = EmailConfirmationHMAC(email)
        send_register_mail.delay(user, confirmation.key)
        return user

class LoginUserView(LoginView):
    queryset = ''

    def post(self, request, *args, **kwargs):
        self.request = request
        self.serializer = self.get_serializer(data=self.request.data,
                                              context={'request': request})
        self.serializer.is_valid(raise_exception=True)

        self.login()
        return self.get_response()


class VerifyUserEmailView(VerifyEmailView):

    def post(self, request, *args, **kwargs):
        serializer = self.get_serializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        self.kwargs['key'] = serializer.validated_data['key']
        confirmation = self.get_object()
        confirmation.confirm(self.request)
        return Response({'detail': _('ok')}, status=status.HTTP_200_OK)


class PasswordResetUserView(views.APIView):
    permission_classes = (permissions.AllowAny,)

    def post(self, request, *args, **kwargs):
        email = request.data.get("email", None)

        try:
            user = User.objects.get(email=email)
        except User.DoesNotExist:
            raise exceptions.NotAcceptable(_("please enter correct email."))
        send_reset_password_email.delay(user)
        return Response({"detail": _("Password reset has been sent.")}, 
                        status=status.HTTP_200_OK)


class PasswordResetConfirmUserView(generics.GenericAPIView):
    serializer_class = PasswordResetConfirmSerializer
    permission_classes = (permissions.AllowAny,)

    @sensitive_post_parameters_m
    def dispatch(self, *args, **kwargs):
        return super(PasswordResetConfirmUserView, self).dispatch(*args, **kwargs)

    def post(self, request, *args, **kwargs):
        serializer = self.get_serializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response({"detail": _("Password has been reset with the new password.")})


class PasswordUserChangeView(PasswordChangeView):

    def post(self, request, *args, **kwargs):
        serializer = self.get_serializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response({"detail": _("New password has been Changed.")})


class UserDetailsAPIView(generics.RetrieveUpdateAPIView):
    serializer_class = UserDetailsSerializer
    permission_classes = (permissions.IsAuthenticated,)

    def get_object(self):
        return self.request.user

    def get_queryset(self):
        return get_user_model().objects.none()



import smtplib
from email.mime.text import MIMEText
from django.conf import settings
try:
    from celery import shared_task
except ImportError:
    raise ImportError('you need to install celery and setup celery configuration')


from django.utils.http import urlsafe_base64_encode
from django.contrib.auth.tokens import default_token_generator
from django.utils.encoding import force_bytes

url = getattr(settings, 'FRONT_URL', '')


@shared_task
def send_register_mail(user, key):
    body = """
    Hello %s

    Confirmation Mail: %s

    You can see more details in this link: %s/account-confirm-email/%s <br><br>

    Thank you <br><br>
    <p>
    """% (user.first_name, user.email, url, key)
    
    subject = "Registeration Confirmation Mail"
    recipients = [user.email]
    
    try:
        send_email(body, subject, recipients, 'html')
        return "Email Is Sent"
    except Exception as e:
        print("Email not sent ", e)

@shared_task
def send_reset_password_email(user):
    body = """
    hello %s,

    Reset Mail Link : %s/%s/%s

    """ %(user.username, url, urlsafe_base64_encode(force_bytes(user.pk)),default_token_generator.make_token(user))
    subject = "Reset password Mail"
    recipients = [user.email]
    try:
        send_email(body, subject, recipients, 'html')
        return "Email Is Sent"
    except Exception as e:
        print("Email not sent ", e)



def send_email(body, subject, recipients, body_type='plain'):
    session = smtplib.SMTP('smtp.gmail.com', getattr(settings, 'EMAIL_PORT', 587))
    session.starttls()
    session.login(getattr(settings, 'EMAIL_HOST_USER', None), getattr(settings, 'EMAIL_HOST_PASSWORD', None))
    sender = getattr(settings, 'EMAIL_HOST_USER', None)
    msg = MIMEText(body, body_type)
    msg['subject'] = subject
    msg['From'] = sender
    msg['To'] = ", ".join(recipients)
    session.sendmail(sender, recipients, msg.as_string())




"""URL Configuration"""
from django.urls import path, include
from . import views
from rest_auth.views import LogoutView



urlpatterns = [
    path('user/', views.UserDetailsAPIView.as_view(), name='rest_user_details'),
    path('login/', views.LoginUserView.as_view(), name='account_login'),
    path('password/change/', views.PasswordUserChangeView.as_view(), name='rest_password_change'),
    path('password/reset/', views.PasswordResetUserView.as_view(), name='rest_password_reset'),
    path('', include('rest_auth.urls')),
    path('registration/', views.RegisterUserView.as_view(), name='account_signup'),
    path('rest-auth/registration/', include('rest_auth.registration.urls')),
    path('account-confirm-email/<str:key>/', views.VerifyUserEmailView.as_view(), name='account_confirm_email'),
    path('password/reset/confirm/<str:uid>/<str:token>/', views.PasswordResetConfirmUserView.as_view(), name='rest_password_reset_confirm'),
    path('logout/', LogoutView.as_view(), name='rest_logout'),


]


#!/usr/bin/env python

import os
from setuptools import setup, find_packages

here = os.path.abspath(os.path.dirname(__file__))
README = open(os.path.join(here, 'README.md')).read()


setup(
    name='django-drf-auth',
    version='0.1',
    packages=find_packages(),
    description='REST API endpoints for Authentication and Registration',
    long_description=README,
    long_description_content_type="text/markdown",
    author='Thomas Adel',
    author_email='thomas.adel31@gmail.com',
    url='https://github.com/thomas545/django-drf-auth/',
    keywords='django drf auth rest-framework-authentication api',
    zip_safe=False,
    license='MIT',
    install_requires=[
        'Django>=2.0',
        'djangorestframework>=3.1.3',
        'six>=1.9.0',
        'django-allauth>=0.34.0',
        'django-rest-auth>=0.9.2',
        'djangorestframework-jwt==1.11.0',
        'celery>=4.0',
    ],
    include_package_data=True,
    classifiers=[
        'Framework :: Django',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'Operating System :: OS Independent',
        'Topic :: Software Development',
    ],
)